package geniusweb.simplerunner.gui;

import java.awt.BorderLayout;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.logging.Level;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import geniusweb.simplerunner.ClassPathConnectionFactory;
import geniusweb.simplerunner.Runner;
import tudelft.utilities.logging.Reporter;

/**
 * Simple GUI allowing users to create the settings interactively, much like on
 * the runserver
 *
 */
public class GUI extends JPanel {
	private JTextArea resultarea = new JTextArea(
			"Results will appear here after run", 9, 40);
	private final SettingsPanel settingsPanel;

	public GUI() {
		setLayout(new BorderLayout());
		settingsPanel = new SettingsPanel();
		add(settingsPanel, BorderLayout.CENTER);
		add(startPanel(), BorderLayout.SOUTH);
	}

	private JPanel startPanel() {
		JPanel startpanel = new JPanel();
		startpanel.setLayout(new BorderLayout());
		JButton startbutton = new JButton("Start Session");
		startpanel.add(startbutton, BorderLayout.NORTH);
		startpanel.add(new JScrollPane(resultarea), BorderLayout.CENTER);

		startbutton.addActionListener(evt -> runsession());
		return startpanel;
	}

	private void runsession() {
		resultarea.setText("");
		Reporter myreporter = new Reporter() {

			@Override
			public void log(Level level, String msg) {
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						resultarea.append("[" + level + "] " + msg + "\n");
					}
				});
			}

			@Override
			public void log(Level level, String msg, Throwable thrown) {
				SwingUtilities.invokeLater(new Runnable() {

					@Override
					public void run() {
						resultarea
								.append("[" + level + "] throws " + msg + "\n");
						StringWriter errors = new StringWriter();
						thrown.printStackTrace(new PrintWriter(errors));
						resultarea.append(errors.toString() + "\n");
					}
				});
			}
		};
		Runner runner = new Runner(settingsPanel.getSettings(),
				new ClassPathConnectionFactory(), myreporter, 0);
		runner.run();
	}

	public static void main(String[] args) {
		JFrame frame = new JFrame("GeniusWeb SimpleRunner GUI");
		frame.add(new GUI());
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}
}
